import numpy as np
import ase
from scipy.interpolate import UnivariateSpline
from  ase import io
import os

import matplotlib.pyplot as plt

def fit_EXAFS(coords_file_name,atoms,E_shift_min,E_shift_max,x_min,x_max,grid_density,abs_atom,atomic_number,edge_shift,scanning_step,exp_file_name):
    n_points = int((E_shift_max-E_shift_min)/scanning_step)
    for i,ii in enumerate(reversed(atoms)):
        if atoms.get_chemical_symbols()[i] == abs_atom:
            first_atom=i+1
            break
    print first_atom
    file_names_ifeffit = [
            'calc.iff',
            'calc2.iff'
            ]
    shifts = []
    x_data_exp = np.loadtxt('fe_chi_k3_exp.DAT',usecols=range(0,1))
    y_data_exp = np.loadtxt('fe_chi_k3_exp.DAT',usecols=range(1,2))
    y_func_exp = UnivariateSpline(x_data_exp,y_data_exp,k=3,s=0.025)
    x_k = np.linspace(x_min,x_max,grid_density)
    shift_array = np.linspace(E_shift_min,E_shift_max,n_points+1)
    deviations = []
    plt.plot(x_data_exp,y_data_exp,label = 'exp')
    for k,kk in enumerate(shift_array):
        with open ('calc.iff','r+') as f:
            content=f.readlines()
            for i,ii in enumerate(content):
                if 'fek_chi.k = (((fe_chi{0}.k)^2)/0.2624683)'.format(first_atom) in ii:
                    content[i] = 'fek_chi.k = (((fe_chi{0}.k)^2)/0.2624683) + {1} \n'.format(first_atom,kk)
        with open('calc.iff','r+') as f:
            f.writelines(content)
        os.system('ifeffit calc.iff')
        os.system('ifeffit calc2.iff')
        x_data_calc = np.loadtxt('fe_chi_k3.DAT',usecols=range(0,1))
        y_data_calc = np.loadtxt('fe_chi_k3.DAT',usecols=range(1,2))
        y_func_calc = UnivariateSpline(x_data_calc,y_data_calc,k=3,s=0.025)
        s = np.sum((y_func_exp(x_k)-y_func_calc(x_k))**2)
        deviations.append(s)
        print kk,s
        plt.plot(x_k,y_func_calc(x_k),label = '{0}'.format(kk))
        plt.xlim(2.,11.)
        plt.ylim(-8.,11.)
        plt.xlabel('k / inverse angstrom')
        plt.ylabel('EXAFS intensity/ a.u.')
    plt.legend()
    plt.show()
    #print deviations
    best_shift = shift_array[np.array(deviations).argmin()]
    print np.array(deviations).min(), best_shift 
    plt.plot(shift_array, np.array(deviations))
    plt.xlabel('Edge Shift, eV')
    plt.ylabel('RMSD, eV')
    plt.show()
    with open ('calc.iff','r+') as f:
        content=f.readlines()
        for i,ii in enumerate(content):
            if 'fek_chi.k = (((fe_chi83.k)^2)/0.2624683)' in ii:
                content[i] = 'fek_chi.k = (((fe_chi{0}.k)^2)/0.2624683) + {1} \n'.format(first_atom,best_shift)
    with open('calc.iff','r+') as f:
        f.writelines(content)
    os.system('ifeffit calc.iff')
    os.system('ifeffit calc2.iff')
    x_data_calc = np.loadtxt('fe_chi_k3.DAT',usecols=range(0,1))
    y_data_calc = np.loadtxt('fe_chi_k3.DAT',usecols=range(1,2))
    y_func_calc = UnivariateSpline(x_data_calc,y_data_calc,k=3,s=0.025)
    plt.plot(x_data_exp,y_data_exp)
    plt.plot(x_k,y_func_calc(x_k))
    plt.xlim(x_min,x_max)
    plt.ylim(-8.,11.)
    plt.xlabel('k / inverse angstrom')
    plt.ylabel('EXAFS intensity/ a.u.')
    plt.show()

    return None

def EXAFS_wrapper(atoms,E_shift_min,E_shift_max,kmin_k,kmax_k,grid_density,abs_atom,\
        atomic_number,edge_shift,file_names,file_names_ifeffit,kmin,kmax,exp_file_name):
    abs_atoms_line = ''
    for i,ii in enumerate(atoms):
        if atoms.get_chemical_symbols()[i] == abs_atom:
            abs_atoms_line+= str(i+1) + ' '
    print abs_atoms_line

    with open ('xyz_to_feff.f90','r+') as f:
        content=f.readlines()
        for i,ii in enumerate(content):
            if 'write(2,*)\'0 ' in ii:
                content[i] = 'write(2,*)\'0  {0}  {1}\' \n'.format(atomic_number,abs_atom)
    with open('xyz_to_feff.f90','r+') as f:
            f.writelines(content)

    with open ('process.iff','r+') as f:
        content=f.readlines()
        for i,ii in enumerate(content):
            if 'newdata.k = range(' in ii:
                content[i] = 'newdata.k = range({0},{1},0.05) \n'.format(kmin_k,kmax_k)
    with open('process.iff','r+') as f:
            f.writelines(content)

    os.system('./compile_xyzfeff')
    for j,jj in enumerate(file_names):
        with open ('{0}'.format(jj),'r+') as f:
            content=f.readlines()
            for i,ii in enumerate(content):
                if 'for abs in' in ii:
                    content[i] = 'for abs in {0} ;do\n'.format(abs_atoms_line)
        with open('{0}'.format(jj),'w') as f:
                f.writelines(content)
        os.system('./{0}'.format(jj))    

    with open ('calc.iff','r+') as f:
        content=f.readlines()
        for i,ii in enumerate(content):
            if 'fek_chi.k = (((fe_chi83.k)^2)/0.2624683)' in ii:
                content[i] = 'fek_chi.k = (((fe_chi83.k)^2)/0.2624683)+ {0} \n'.format(edge_shift)
    with open('calc.iff','r+') as f:
            f.writelines(content)

    with open ('calc2.iff','r+') as f:
        content=f.readlines()
        for i,ii in enumerate(content):
            if 'fftf(newfe.chi,' in ii:
                content[i] = 'fftf(newfe.chi, k=newfe.k, kmin={0}, kmax={1}, dk=1.1513, kweight=3,\
                        kwindow=\'hanning\',group=newfeFT) \n'.format(kmin,kmax)
    with open('calc2.iff','r+') as f:
            f.writelines(content)
    for i,ii in enumerate(file_names_ifeffit):
        os.system('ifeffit {0}'.format(ii))
    x_data_exp = np.loadtxt('fe_chi_k3_exp.DAT',usecols=range(0,1))
    y_data_exp = np.loadtxt('fe_chi_k3_exp.DAT',usecols=range(1,2))
    E_axis = np.zeros(len(x_data_exp))
    Chi_axis = np.zeros(len(y_data_exp))
    for e, ee in enumerate(x_data_exp):
        if ee == 0.:
            E_axis[e] = 0.
            Chi_axis[e] = 0.
        else:
            E_axis[e] = x_data_exp[e]**2/0.2624683
            Chi_axis[e] = y_data_exp[e]/x_data_exp[e]**3
    np.savetxt('exp_chi_vs_E.dat',np.hstack([E_axis.reshape(-1,1),Chi_axis.reshape(-1,1)]))
    with open ('calc2_exp.iff','r+') as f:
        content=f.readlines()
        for i,ii in enumerate(content):
            if 'fftf(newfe.chi,' in ii:
                content[i] = 'fftf(newfe.chi, k=newfe.k, kmin={0}, kmax={1}, dk=1.1513, kweight=3,\
                        kwindow=\'hanning\',group=newfeFT) \n'.format(kmin,kmax)
    with open('calc2_exp.iff','r+') as f:
            f.writelines(content)
    os.system('ifeffit calc2_exp.iff'.format(ii))
    return None

def build_plots(exp_file_name,k_lower, k_upper,chi_lower,chi_upper,FT_lower,FT_upper,rd_lower,rd_upper):
    import matplotlib.pyplot as plt
    import matplotlib.gridspec as gridspec
    from mpl_toolkits.axes_grid1 import host_subplot
    import mpl_toolkits.axisartist as AA

    k_exp_x = np.loadtxt('{0}'.format(exp_file_name),usecols=range(0,1))
    k_exp_y = np.loadtxt('{0}'.format(exp_file_name),usecols=range(1,2))
    FT_exp_x = np.loadtxt('fe_FT_exp.out',usecols=range(0,1))
    FT_exp_y = np.loadtxt('fe_FT_exp.out',usecols=range(1,2))
    k_x = np.loadtxt('fe_chi_k3.DAT',usecols=range(0,1))
    k_y = np.loadtxt('fe_chi_k3.DAT',usecols=range(1,2))
    FT_x = np.loadtxt('fe_FT.out',usecols=range(0,1))
    FT_y = np.loadtxt('fe_FT.out',usecols=range(1,2))
    fig = plt.figure()
    ax=np.array([])
    ax = np.empty(shape=(2,), dtype=object)
    ax[0] = plt.subplot2grid((1,2), (0,0), colspan=1)
    ax[1] = plt.subplot2grid((1,2), (0,1), colspan=1)
    counter=0
    ax[counter].plot(k_x,k_y,lw=2,color='red',label = 'Calc')
    ax[counter].plot(k_exp_x,k_exp_y,label='Exp',lw=2, color='black')
    ax[counter].set_xlim(k_lower,k_upper)
    ax[counter].legend(loc="upper right",ncol=1,prop={'size':8}).get_frame().set_linewidth(0.0)
    ax[counter].set_ylim(chi_lower,chi_upper)
    ax[counter].set_xlabel(r'k/ $ \AA ^{-1}$', fontsize=8,labelpad=-2)
    ax[counter].set_ylabel(r'EXAFS $\chi k^3$', fontsize=8,labelpad=-8)
    ax[counter].tick_params(axis='x', labelsize=8)
    ax[counter].tick_params(axis='y', labelsize=8)
    ax[counter].text(-0.1, 1.00, 'A', fontsize=12,transform=ax[counter].transAxes, fontweight='bold', va='top', ha='right')
    counter+=1
    ax[counter].plot(FT_x, FT_y,lw=2,color='red',label = 'Calc')
    ax[counter].plot(FT_exp_x, FT_exp_y,lw=2,label='Exp', color='black')
    ax[counter].set_xlim(0.,6.)
    ax[counter].legend(loc="upper right",ncol=1,prop={'size':8}).get_frame().set_linewidth(0.0)
    ax[counter].set_xlabel(r'Reduced Distance / $\AA $', fontsize=8)
    ax[counter].set_ylabel('FT Magnitude / a.u.', fontsize=8)
    ax[counter].tick_params(axis='x', labelsize=8)
    ax[counter].tick_params(axis='y', labelsize=8)
    ax[counter].text(-0.1, 1.00, 'B', fontsize=12,transform=ax[counter].transAxes, fontweight='bold', va='top', ha='right')
    counter+=1

    plt.show()
    fig.set_size_inches(7,3)
    fig.savefig('EXAFS.png',dpi=300, bbox_inches='tight')

def calculate_FT(N,T,name_FT_input,name_FT_output,kmin_FT,kmax_FT,dk):
    def hanning_window(x,kmin_FT,kmax_FT):
        y = np.zeros(len(x))
        lower_n = (np.abs(x-kmin_FT)).argmin()
        upper_n = (np.abs(x-kmax_FT)).argmin()
        N = float(upper_n-lower_n)
        counter=0.
        for i,ii in enumerate(x):
            if ii<k_lower:
                y[i] = 0.
            if ii>=k_lower and ii<kmax_FT:
                y[i]= 0.5*(1-np.cos(2*np.pi*counter/(N-1)))
                counter+=1
            if ii>=kmax_FT:
                y[i] = 0.
        return y
    def ifeffit_hanning_window(x,kmin_FT,kmax_FT,dk):
        y = np.zeros(len(x))
        k1 = kmin_FT-dk/2
        k2 = kmin_FT+dk/2
        k3 = kmax_FT-dk/2
        k4 = kmax_FT+dk/2
        #print k1,k2,k3,k4
        k1n = (np.abs(x-k1)).argmin()
        k2n = (np.abs(x-k2)).argmin()
        k3n = (np.abs(x-k3)).argmin()
        k4n = (np.abs(x-k4)).argmin()
        N = float(k2n-k1n+k4n-k3n)
        print N
        counter=0.
        for i,ii in enumerate(x):
            if ii<k1:
                y[i] = 0.
            if ii>=k1 and ii<k2:
                y[i]= 0.5*(1-np.cos(2*np.pi*counter/(N-1)))
                counter+=1
            if ii>=k2 and ii<k3:
                y[i] = 1.
            if ii>=k3 and ii<k4:
                y[i]= 0.5*(1-np.cos(2*np.pi*counter/(N-1)))
                counter+=1
            if ii>=k4:
                y[i] = 0.
        return y
    def square_window(x,kmin_FT,kmax_FT):
        y = np.zeros(len(x))
        lower_n = (np.abs(x-kmin_FT)).argmin()
        upper_n = (np.abs(x-kmax_FT)).argmin()
        counter=0.
        for i,ii in enumerate(x):
            if ii<kmin_FT:
                y[i] = 0.
            if ii>=kmin_FT and ii<kmax_FT:
                y[i]= 1.
            if ii>=kmax_FT:
                y[i] = 0.
        return y
    from scipy.fftpack import fft
    k_x = np.loadtxt('{0}'.format(name_FT_input),usecols=range(0,1))
    k_y = np.loadtxt('{0}'.format(name_FT_input),usecols=range(1,2))
    y_func=UnivariateSpline(k_x[10:],k_y[10:],k=3,s=0.025)
    x = np.linspace(0.0, N*T, N)
    yf = fft(y_func(x)*ifeffit_hanning_window(x,kmin_FT, kmax_FT,dk))
    xf = np.linspace(0.0, 1.0/(2.0*T), N/2)*np.pi
    yf_final = 2.0/N * yf[0:N/2]*np.conjugate(yf[0:N/2])    
    xf = np.real(xf)
    yf_final  = np.real(yf_final)
    np.savetxt('{0}'.format(name_FT_output),np.hstack([xf.reshape(-1,1),yf_final.reshape(-1,1)]))



