#######################################
#### Script that calcultes EXAFS   ####
#### and fits it to the experiment ####
#### Created by Mikhail Askerka    ####
####   Yale University 10/16       ####
####    askerka.m@gmail.com        ####
#######################################

import numpy as np
import ase
from  ase import io
import os
from os.path import join, isfile, islink, getmtime
import sys, math, re, string, subprocess, numpy
from EXAFS_routines import *
import sys

usage = '\n\
\n\
%s - A Python script that constructs EXAFS spectra in the momentum and reduced distance coordinates and fits it to experimental data\n\
\n\
###########################################\n\
###### Usage: python %s [inputfile] #######\n\
###########################################\n\
  Input file keywords:\n\
\n\
#############   Action options   ######################\n\
build_exafs                           (0 or 1) Build EXAFS spectra using FEFF package \n\
fitting                               (0 or 1) Fit the spectra to the experiment by shifting the EXAFS edge\n\
build_plot                            (0 or 1) Build and save high resolution plots    \n\
do_home_made_FT                       (0 or 1) Do the home made Fourier transform or the Ifeffit Fourier Transform\n\
#############   EXAFS  options   ######################\n\
abs_atom                              (str) Absorbing atom in the EXAFS experiment \n\
atomic_number                         (int) Atomic number of that atom\n\
edge_shift                            (float) Initial guess for the shift edge\n\
n_points                              (int) Grid density for the k-spectrum interpolation\n\
abs_atoms_line                        (str) Always "" not sure why I made this an input\n\
#############   Fitting  options   ####################\n\
E_shift_min                           (float) Starting edge shift for the scan\n\
E_shift_max                           (float) Final edge shift for the scan\n\
scanning_step                         (float) Step for the scan\n\
grid_density                          (int) Grid density for the k-spectrum interpolation\n\
############  Ifeffit FT options   ####################\n\
kmin                                  (float) Lower boundary for the FT window function\n\
kmax                                  (float) Upper boundary for the FT window function\n\
############# Home made FT options ####################\n\
kmin_FT                               (float) Lower boundary for the FT window function\n\
kmax_FT                               (float) Upper boundary for the FT window function\n\
dk                                    (float) Window width. The greater the value, the slower the window decays\n\
T                                     (float) Step size for the new axis in the Fourier Transform\n\
N                                     (int) Number of points on the new axis in the Fourier Transform\n\
name_FT_input                         (array of str) The names of the files with Chi*k^3 vs k data you want to do the FT on\n\
name_FT_output                        (array of str) The names of the output files for the FT on\n\
############   File name options  #####################\n\
file_names                            (array of str) bash files that need to be altered to use FEFF \n\
file_names_ifeffit                    (array of str) bash files that need to be altered to use Ifeffit\n\
coords_file_name                      (str) the name of the file with input coordinates\n\
exp_file_name                         (str) the name of the file with data in Chi*k^3 vs k \n\
############   Plotting options  ######################\n\
k_lower                               (float) Lower boundary for k in the Chi*k^3 vs k plot\n\
k_upper                               (float) Upper boundary for k in the Chi*k^3 vs k plot\n\
chi_lower                             (float) Lower boundary for Chi in the Chi*k^3 vs k plot\n\
chi_upper                             (float) Upper boundary for k in the Chi*k^3 vs k plot\n\
FT_lower                              (float) Lower boundary for the FT magnitude in the FT plot\n\
FT_upper                              (float) Upper boundary for the FT magnitude in the FT plot\n\
rd_lower                              (float) Lower boundary for the reduced distance in the FT plot\n\
rd_upper                              (float) Upper boundary for the reduced distance in the FT plot\n\
\n\
#######################################################\n\
#############  Input File Example  ####################\n\
#######################################################\n\
\n\
#############   Action options   ######################\n\
build_exafs                           1\n\
fitting                               1 \n\
build_plot                            1\n\
do_home_made_FT                       1 \n\
#############   EXAFS  options   ######################\n\
abs_atom                              Mn\n\
atomic_number                         25\n\
edge_shift                            20\n\
n_points                              1000\n\
abs_atoms_line                        ''\n\
#############   Fitting  options   ####################\n\
E_shift_min                           11.\n\
E_shift_max                           12.5\n\
scanning_step                         0.2\n\
grid_density                          1000\n\
############  Ifeffit FT options   ####################\n\
kmin                                  4.0 \n\
kmax                                  10.5\n\
############# Home made FT options ####################\n\
kmin_FT                               4.0 \n\
kmax_FT                               10.5\n\
dk                                    3.\n\
T                                     0.1\n\
N                                     1000\n\
name_FT_input                         fe_chi_k3.DAT chik3_exp_S3.DAT\n\
name_FT_output                        fe_FT.out fe_FT_exp.out\n\
############   File name options  #####################\n\
file_names                            compile_xyzfeff exafs_fit make_calc\n\
file_names_ifeffit                    calc.iff calc2.iff\n\
coords_file_name                      coords_S3.xyz\n\
exp_file_name                         chik3_exp_S3.DAT\n\
############   Plotting options  ######################\n\
k_lower                               2\n\
k_upper                               12.7\n\
chi_lower                             -8.\n\
chi_upper                             8.\n\
FT_lower                              0.\n\
FT_upper                              8.\n\
rd_lower                              0.\n\
rd_upper                              6.\n\
' % (sys.argv[0], sys.argv[0])


if (len(sys.argv) < 2 or len(sys.argv) > 2):
   sys.exit(usage)

script_name = sys.argv[0]
input_filename = sys.argv[1]

################################
##### Create the Defaults ######
################################
abs_atom = "Mn"
atomic_number = "25"
edge_shift=20. #shift of the edge of the EXAFS
just_fitting = 0
fitting = 1 # 1 for True, 0 for False
show_plot = 1 # 1 for True, 0 for False
n_points=1000
E_shift_min = 10.
E_shift_max = +15.
grid_density = 1000
kmin=4.0
kmax=10.5
file_names = [
        'compile_xyzfeff',
        'exafs_fit',
        'make_calc'
        ]
file_names_ifeffit = [
        'calc.iff',
        'calc2.iff'
        ]
################################
##### Read in the Input   ######
################################

input_dic = {}
with open("{0}".format(input_filename)) as f:
    for line in f:
        (key, val) = line.split()[0],line.split()[1:]
        input_dic[(key)] = val

abs_atom = str(input_dic['abs_atom'][0])
atomic_number = str(input_dic['atomic_number'][0])
edge_shift= float(input_dic['edge_shift'][0])
build_exafs = int(input_dic['build_exafs'][0])
fitting = int(input_dic['fitting'][0])
do_home_made_FT = int(input_dic['do_home_made_FT'][0])
build_plot = int(input_dic['build_plot'][0])
n_points=int(input_dic['n_points'][0])
E_shift_min = float(input_dic['E_shift_min'][0])
scanning_step = float(input_dic['scanning_step'][0])
E_shift_max = float(input_dic['E_shift_max'][0])
grid_density = int(input_dic['grid_density'][0])
kmin=float(input_dic['kmin'][0])
kmax=float(input_dic['kmax'][0])
file_names = input_dic['file_names']
file_names_ifeffit = input_dic['file_names_ifeffit']
coords_file_name = str(input_dic['coords_file_name'][0])
exp_file_name = str(input_dic['exp_file_name'][0])
N= int(input_dic['N'][0])
T= float(input_dic['T'][0])
name_FT_input= input_dic['name_FT_input']
name_FT_output= input_dic['name_FT_output']
kmin_FT= float(input_dic['kmin_FT'][0])
kmax_FT= float(input_dic['kmax_FT'][0])
dk= float(input_dic['dk'][0])

k_lower= float(input_dic['k_lower'][0])
k_upper= float(input_dic['k_upper'][0])
chi_lower= float(input_dic['chi_lower'][0])
chi_upper= float(input_dic['chi_upper'][0])
FT_lower= float(input_dic['FT_lower'][0])
FT_upper= float(input_dic['FT_upper'][0])
rd_lower= float(input_dic['rd_lower'][0])
rd_upper= float(input_dic['rd_upper'][0])

##################################
#### END OF THE INPUT SECTION ####
##################################

kmin_k = 0.
atoms = ase.io.read(coords_file_name)
kmax_k = np.loadtxt(exp_file_name)[-1,0]
os.system('cp {0} coords.xyz'.format( coords_file_name))

action_array = [build_exafs,fitting,build_plot,do_home_made_FT]

if action_array==[0,1,1,0]: 
    fit_EXAFS(coords_file_name,atoms,E_shift_min,E_shift_max,kmin_k,kmax_k,grid_density,abs_atom,atomic_number,edge_shift,scanning_step,exp_file_name)
    build_plots(exp_file_name,k_lower, k_upper,chi_lower,chi_upper,FT_lower,FT_upper,rd_lower,rd_upper)
elif action_array == [0,0,1,0]:
    build_plots(exp_file_name,k_lower, k_upper,chi_lower,chi_upper,FT_lower,FT_upper,rd_lower,rd_upper)
elif action_array == [1,0,0,0]:
    EXAFS_wrapper(atoms,E_shift_min,E_shift_max,kmin_k,kmax_k,grid_density,\
            abs_atom,atomic_number,edge_shift,file_names,file_names_ifeffit,kmin,kmax,exp_file_name) 
elif action_array == [1,1,0,0]:
    EXAFS_wrapper(atoms,E_shift_min,E_shift_max,kmin_k,kmax_k,grid_density,\
            abs_atom,atomic_number,edge_shift,file_names,file_names_ifeffit,kmin,kmax,exp_file_name) 
    fit_EXAFS(coords_file_name,atoms,E_shift_min,E_shift_max,kmin_k,kmax_k,grid_density,abs_atom,atomic_number,edge_shift,scanning_step,exp_file_name)
elif action_array == [1,0,1,0]:
    EXAFS_wrapper(atoms,E_shift_min,E_shift_max,kmin_k,kmax_k,grid_density,\
            abs_atom,atomic_number,edge_shift,file_names,file_names_ifeffit,kmin,kmax,exp_file_name) 
    build_plots(exp_file_name,k_lower, k_upper,chi_lower,chi_upper,FT_lower,FT_upper,rd_lower,rd_upper)
elif action_array == [1,1,1,0]:
    EXAFS_wrapper(atoms,E_shift_min,E_shift_max,kmin_k,kmax_k,grid_density,\
            abs_atom,atomic_number,edge_shift,file_names,file_names_ifeffit,kmin,kmax,exp_file_name) 
    fit_EXAFS(coords_file_name,atoms,E_shift_min,E_shift_max,kmin_k,kmax_k,grid_density,abs_atom,atomic_number,edge_shift,scanning_step,exp_file_name)
    build_plots(exp_file_name,k_lower, k_upper,chi_lower,chi_upper,FT_lower,FT_upper,rd_lower,rd_upper)
elif action_array == [1,1,1,1]:
    EXAFS_wrapper(atoms,E_shift_min,E_shift_max,kmin_k,kmax_k,grid_density,\
            abs_atom,atomic_number,edge_shift,file_names,file_names_ifeffit,kmin,kmax,exp_file_name) 
    fit_EXAFS(coords_file_name,atoms,E_shift_min,E_shift_max,kmin_k,kmax_k,grid_density,abs_atom,atomic_number,edge_shift,scanning_step,exp_file_name)
    for i,ii in enumerate(name_FT_input):
        calculate_FT(N,T,name_FT_input[i],name_FT_output[i],kmin_FT,kmax_FT,dk)
    build_plots(exp_file_name,k_lower, k_upper,chi_lower,chi_upper,FT_lower,FT_upper,rd_lower,rd_upper)
elif action_array == [0,0,1,1]:
    for i,ii in enumerate(name_FT_input):
        calculate_FT(N,T,name_FT_input[i],name_FT_output[i],kmin_FT,kmax_FT,dk)
    build_plots(exp_file_name,k_lower, k_upper,chi_lower,chi_upper,FT_lower,FT_upper,rd_lower,rd_upper)
else:
    print "We do not have an option you are requesting!"




